﻿using System;

namespace AC.Input
{
	/// <summary>
	/// The state of a button. This can be a Mouse, Keyboard or GamePad button.
	/// </summary>
	public enum ButtonState : int
	{
		/// <summary>
		/// The button is currently not pressed, nor was it pressed in the previous tick.
		/// </summary>
		NotPressed = 0,

		/// <summary>
		/// The button was just pressed in this tick. The state before was either
		/// Released or NotPressed.
		/// </summary>
		JustPressed = 1,

		/// <summary>
		/// The button is currently pressed and held down. This means the previous
		/// state was JustPressed and the button is at least pressed the second tick
		/// now.
		/// </summary>
		Pressed = 2,

		/// <summary>
		/// The button was released in this tick. The previous state was either
		/// JustPressed or Pressed. This state only lasts for one tick and the next
		/// tick we get either the JustPressed or NotPressed state.
		/// </summary>
		Released = 3,
	}
}
