﻿using System;
using AC.Input.Platforms;

namespace AC.Input
{
	public class GamePad
	{
		#region Static
		public static GamePad One
		{
			get;
			private set;
		}

		public static GamePad Two
		{
			get;
			private set;
		}

		public static GamePad Three
		{
			get;
			private set;
		}

		public static GamePad Four
		{
			get;
			private set;
		}
		#endregion

		#region Public
		public int Index
		{
			get;
			private set;
		}

		public bool IsConnected
		{
			get;
			private set;
		}

		#region ThumbStickLeftX
		private float privateThumbStickLeftX;
		public float ThumbStickLeftX
		{
			get
			{
				return privateThumbStickLeftX;
			}
		}
		#endregion

		#region ThumbStickLeftY
		private float privateThumbStickLeftY;
		public float ThumbStickLeftY
		{
			get
			{
				return privateThumbStickLeftY;
			}
		}
		#endregion

		#region ThumbStickRightX
		private float privateThumbStickRightX;
		public float ThumbStickRightX
		{
			get
			{
				return privateThumbStickRightX;
			}
		}
		#endregion

		#region ThumbStickRightY
		private float privateThumbStickRightY;
		public float ThumbStickRightY
		{
			get
			{
				return privateThumbStickRightY;
			}
		}
		#endregion

		#region TriggerLeft
		private float privateTriggerLeft;
		public float TriggerLeft
		{
			get
			{
				return privateTriggerLeft;
			}
		}
		#endregion

		#region TriggerRight
		private float privateTriggerRight;
		public float TriggerRight
		{
			get
			{
				return privateTriggerRight;
			}
		}
		#endregion

		public GamePadDPad DPad
		{
			get;
			private set;
		}

		public GamePadButtons Buttons
		{
			get;
			private set;
		}
		#endregion

		#region Constructor
		static GamePad()
		{
			One = new GamePad(1);
			Two = new GamePad(2);
			Three = new GamePad(3);
			Four = new GamePad(4);
		}

		private GamePad(int setIndex)
		{
			Index = setIndex;
			DPad = new GamePadDPad();
			Buttons = new GamePadButtons();
		}
		#endregion

		#region Update
		internal void Update()
		{
			IsConnected = BaseContext.Current.CheckGamePadConnection(Index - 1);

			bool[] dpadButtons = new bool[4];
			bool[] buttons = new bool[11];

			if (IsConnected == false)
			{
				privateThumbStickLeftX = 0f;
				privateThumbStickLeftY = 0f;
				privateThumbStickRightX = 0f;
				privateThumbStickRightY = 0f;
				privateTriggerLeft = 0f;
				privateTriggerRight = 0f;
			}
			else
			{
				BaseContext.Current.SetGamePadValues(Index - 1,
					out privateThumbStickLeftX, out privateThumbStickLeftY,
					out privateThumbStickRightX, out privateThumbStickRightY,
					out privateTriggerLeft, out privateTriggerRight,
					ref dpadButtons, ref buttons);
			}

			DPad.Top = UpdateState(DPad.Top, dpadButtons[0]);
			DPad.Bottom = UpdateState(DPad.Bottom, dpadButtons[1]);
			DPad.Left = UpdateState(DPad.Left, dpadButtons[2]);
			DPad.Right = UpdateState(DPad.Right, dpadButtons[3]);

			Buttons.A = UpdateState(Buttons.A, buttons[0]);
			Buttons.B = UpdateState(Buttons.B, buttons[1]);
			Buttons.X = UpdateState(Buttons.X, buttons[2]);
			Buttons.Y = UpdateState(Buttons.Y, buttons[3]);
			Buttons.Back = UpdateState(Buttons.Back, buttons[4]);
			Buttons.Start = UpdateState(Buttons.Start, buttons[5]);
			Buttons.ShoulderLeft = UpdateState(Buttons.ShoulderLeft, buttons[6]);
			Buttons.ShoulderRight = UpdateState(Buttons.ShoulderRight, buttons[7]);
			Buttons.ThumbStickLeft = UpdateState(Buttons.ThumbStickLeft, buttons[8]);
			Buttons.ThumbStickRight = UpdateState(Buttons.ThumbStickRight, buttons[9]);
			Buttons.Big = UpdateState(Buttons.Big, buttons[10]);
		}
		#endregion

		#region UpdateState
		private ButtonState UpdateState(ButtonState oldState, bool isPressed)
		{
			if ((oldState == ButtonState.NotPressed ||
				oldState == ButtonState.Released) &&
				isPressed)
			{
				return ButtonState.JustPressed;
			}

			if (oldState == ButtonState.JustPressed &&
				isPressed)
			{
				return ButtonState.Pressed;
			}

			if ((oldState == ButtonState.JustPressed ||
				oldState == ButtonState.Pressed) &&
				isPressed == false)
			{
				return ButtonState.Released;
			}

			if (oldState == ButtonState.Released &&
				isPressed == false)
			{
				oldState = ButtonState.NotPressed;
			}

			return oldState;
		}
		#endregion
	}
}
