using System;
using System.Collections.Generic;
using AC.Input.Platforms;

namespace AC.Input
{
	public class Keyboard
	{
		#region Static
		public static Keyboard Instance
		{
			get;
			private set;
		}
		#endregion

		#region Private
		private ButtonState[] currentStates;
		private List<string> keyNamesCache;
		#endregion

		#region Constructor
		static Keyboard()
		{
			Instance = new Keyboard();
		}

		private Keyboard()
		{
			string[] names = Enum.GetNames(typeof(Keys));
			keyNamesCache = new List<string>(names);

			currentStates = new ButtonState[names.Length];
		}
		#endregion

		#region GetState
		public ButtonState GetState(Keys key)
		{
			return currentStates[keyNamesCache.IndexOf(key.ToString())];
		}
		#endregion

		#region IsKeyDown
		public bool IsKeyDown(Keys key)
		{
			ButtonState state = currentStates[keyNamesCache.IndexOf(key.ToString())];
			return state == ButtonState.JustPressed ||
				state == ButtonState.Pressed;
		}
		#endregion

		#region IsKeyUp
		public bool IsKeyUp(Keys key)
		{
			ButtonState state = currentStates[keyNamesCache.IndexOf(key.ToString())];
			return state == ButtonState.NotPressed ||
				state == ButtonState.Released;
		}
		#endregion

		#region Update
		internal void Update()
		{
			List<Keys> pressedKeys = new List<Keys>(
				BaseContext.Current.GetPressedKeys());

			List<Keys> releasedKeys = new List<Keys>(
				BaseContext.Current.GetReleasedKeys());

			for (int index = 0; index < currentStates.Length; index++)
			{
				Keys key = (Keys)Enum.Parse(typeof(Keys), keyNamesCache[index]);
				ButtonState currentState = currentStates[index];
				bool isReleased = releasedKeys.Contains(key);
				bool isPressed = pressedKeys.Contains(key);

				if (currentState == ButtonState.JustPressed &&
					isReleased == false)
				{
					currentStates[index] = ButtonState.Pressed;
					continue;
				}

				if (isPressed &&
					isReleased == false)
				{
					if (currentState != ButtonState.Pressed)
					{
						currentStates[index] = ButtonState.JustPressed;
					}
					continue;
				}

				if (isReleased)
				{
					currentStates[index] = ButtonState.Released;
				}
				else if (currentState == ButtonState.Released)
				{
					currentStates[index] = ButtonState.NotPressed;
				}
			}
		}
		#endregion
	}
}
