﻿using System;
using System.Drawing;
using AC.Input.Platforms;

namespace AC.Input
{
	public class Mouse
	{
		#region Static
		public static Mouse Instance
		{
			get;
			private set;
		}
		#endregion

		#region Public
		#region LeftButton
		private ButtonState privateLeftButton;
		public ButtonState LeftButton
		{
			get
			{
				return privateLeftButton;
			}
		}
		#endregion

		#region MiddleButton
		private ButtonState privateMiddleButton;
		public ButtonState MiddleButton
		{
			get
			{
				return privateMiddleButton;
			}
		}
		#endregion

		#region RightButton
		private ButtonState privateRightButton;
		public ButtonState RightButton
		{
			get
			{
				return privateRightButton;
			}
		}
		#endregion

		#region X1Button
		private ButtonState privateX1Button;
		public ButtonState X1Button
		{
			get
			{
				return privateX1Button;
			}
		}
		#endregion

		#region X2Button
		private ButtonState privateX2Button;
		public ButtonState X2Button
		{
			get
			{
				return privateX2Button;
			}
		}
		#endregion

		#region ScrollMovement
		private int privateScrollMovement;
		public int ScrollMovement
		{
			get
			{
				return privateScrollMovement;
			}
		}
		#endregion

		#region X
		public int X
		{
			get;
			private set;
		}
		#endregion

		#region Y
		public int Y
		{
			get;
			private set;
		}
		#endregion

		#region XMovement
		public int XMovement
		{
			get;
			private set;
		}
		#endregion

		#region YMovement
		public int YMovement
		{
			get;
			private set;
		}
		#endregion

		#region Position
		public Point Position
		{
			get;
			private set;
		}
		#endregion

		#region Movement
		public Point Movement
		{
			get;
			private set;
		}
		#endregion
		#endregion

		#region Constructor
		static Mouse()
		{
			Instance = new Mouse();
		}

		private Mouse()
		{
			privateLeftButton = ButtonState.NotPressed;
			privateMiddleButton = ButtonState.NotPressed;
			privateRightButton = ButtonState.NotPressed;
			privateX1Button = ButtonState.NotPressed;
			privateX2Button = ButtonState.NotPressed;
		}
		#endregion

		#region Update
		internal void Update()
		{
			bool[] pressed;
			bool[] released;

			int newX;
			int newY;
			BaseContext.Current.SetMouseValues(out newX, out newY, out pressed,
				out released, out privateScrollMovement);
			XMovement = newX - X;
			YMovement = newY - Y;
			X = newX;
			Y = newY;
			Position = new Point(X, Y);
			Movement = new Point(XMovement, YMovement);

			UpdateState(ref privateLeftButton, pressed[0], released[0]);
			UpdateState(ref privateMiddleButton, pressed[1], released[1]);
			UpdateState(ref privateRightButton, pressed[2], released[2]);
			UpdateState(ref privateX1Button, pressed[3], released[3]);
			UpdateState(ref privateX2Button, pressed[4], released[4]);
		}
		#endregion

		#region UpdateState
		private void UpdateState(ref ButtonState oldState, bool isPressed,
			bool isReleased)
		{
			if (oldState == ButtonState.JustPressed &&
				isReleased == false)
			{
				oldState = ButtonState.Pressed;
				return;
			}

			if (isPressed)
			{
				if (oldState != ButtonState.Pressed)
				{
					oldState = ButtonState.JustPressed;
				}
				return;
			}

			if (isReleased)
			{
				oldState = ButtonState.Released;
			}
			else if (oldState == ButtonState.Released)
			{
				oldState = ButtonState.NotPressed;
			}
		}
		#endregion

		#region SetPosition
		public void SetPosition(int x, int y)
		{
			BaseContext.Current.SetMousePosition(x, y);
		}
		#endregion
	}
}
