using System;
using AC.Input.Platforms.Linux;
using AC.Input.Platforms.Windows;

namespace AC.Input.Platforms
{
	internal abstract class BaseContext : IDisposable
	{
		#region Public
		public static BaseContext Current
		{
			get;
			protected set;
		}

		private IntPtr privateWindowHandle;
		public IntPtr WindowHandle
		{
			get
			{
				return privateWindowHandle;
			}
			set
			{
				if (value != privateWindowHandle)
				{
					privateWindowHandle = value;
					WindowHandleChanged();
				}
			}
		}
		#endregion

		#region Constructor
		static BaseContext()
		{
			if (Platform.OnWindows)
			{
				Current = new WindowsContext();
			}
			else if (Platform.OnLinux)
			{
				Current = new LinuxContext();
			}
			else if (Platform.OnMacOS)
			{
				Current = null;
			}
		}

		protected BaseContext()
		{
		}
		#endregion

		#region Dispose (virtual)
		public virtual void Dispose()
		{
		}
		#endregion

		protected abstract void WindowHandleChanged();

		public abstract bool CheckGamePadConnection(int index);

		public abstract void SetMousePosition(int x, int y);

		public abstract void SetGamePadValues(int index,
			out float thumbStickLeftX, out float thumbStickLeftY,
			out float thumbStickRightX, out float thumbStickRightY,
			out float triggerLeft, out float triggerRight,
			ref bool[] dpadButtons, ref bool[] buttons);

		public abstract void SetMouseValues(out int X, out int Y,
			out bool[] pressed, out bool[] released, out int scrollMovement);

		public abstract Keys[] GetPressedKeys();
		public abstract Keys[] GetReleasedKeys();

		public abstract void Update();
	}
}
