using System;
using System.Runtime.InteropServices;

namespace AC.Input.Platforms.Linux
{
	[StructLayout(LayoutKind.Sequential, Pack = 2)]
    internal struct XColor
    {
        internal IntPtr pixel;
        internal ushort red;
        internal ushort green;
        internal ushort blue;
        internal byte flags;
        internal byte pad;
    }
		
	internal enum XEventName : int
	{
		KeyPress = 2,
		KeyRelease = 3,
		ButtonPress = 4,
		ButtonRelease = 5,
		MotionNotify = 6,
		EnterNotify = 7,
		LeaveNotify = 8,
		FocusIn = 9,
		FocusOut = 10,
		KeymapNotify = 11,
		Expose = 12,
		GraphicsExpose = 13,
		NoExpose = 14,
		VisibilityNotify = 15,
		CreateNotify = 16,
		DestroyNotify = 17,
		UnmapNotify = 18,
		MapNotify = 19,
		MapRequest = 20,
		ReparentNotify = 21,
		ConfigureNotify = 22,
		ConfigureRequest = 23,
		GravityNotify = 24,
		ResizeRequest = 25,
		CirculateNotify = 26,
		CirculateRequest = 27,
		PropertyNotify = 28,
		SelectionClear = 29,
		SelectionRequest = 30,
		SelectionNotify = 31,
		ColormapNotify = 32,
		ClientMessage = 33,
		MappingNotify = 34,

		LASTEvent
	}
	
	[StructLayout(LayoutKind.Sequential)]
	internal struct XAnyEvent
	{
		public XEventName type;
		public IntPtr serial;
		public bool send_event;
		public IntPtr display;
		public IntPtr window;
	}
	
	[StructLayout(LayoutKind.Explicit)]
	internal struct XEvent
	{
		[FieldOffset(0)]
		public XEventName type;
		[FieldOffset(0)]
		public XAnyEvent AnyEvent;
		[FieldOffset(0)]
		public XKeyEvent KeyEvent;
		[FieldOffset(0)]
		public XButtonEvent ButtonEvent;
		[FieldOffset(0)]
		public XMotionEvent MotionEvent;
		[FieldOffset(0)]
		public IntPtr xcrossing;
		[FieldOffset(0)]
		public IntPtr xfocus;
		[FieldOffset(0)]
		public IntPtr xexpose;
		[FieldOffset(0)]
		public IntPtr xgraphicsexpose;
		[FieldOffset(0)]
		public IntPtr xnoexpose;
		[FieldOffset(0)]
		public IntPtr xvisibility;
		[FieldOffset(0)]
		public IntPtr xcreatewindow;
		[FieldOffset(0)]
		public IntPtr xdestroywindow;
		[FieldOffset(0)]
		public IntPtr xunmap;
		[FieldOffset(0)]
		public IntPtr xmap;
		[FieldOffset(0)]
		public IntPtr xmaprequest;
		[FieldOffset(0)]
		public IntPtr xreparent;
		[FieldOffset(0)]
		public IntPtr xconfigure;
		[FieldOffset(0)]
		public IntPtr xgravity;
		[FieldOffset(0)]
		public IntPtr xresizerequest;
		[FieldOffset(0)]
		public IntPtr xconfigurerequest;
		[FieldOffset(0)]
		public IntPtr xcirculate;
		[FieldOffset(0)]
		public IntPtr xcirculaterequest;
		[FieldOffset(0)]
		public IntPtr xproperty;
		[FieldOffset(0)]
		public IntPtr xselectionclear;
		[FieldOffset(0)]
		public IntPtr xselectionrequest;
		[FieldOffset(0)]
		public IntPtr xselection;
		[FieldOffset(0)]
		public IntPtr xcolormap;
		[FieldOffset(0)]
		public IntPtr xclient;
		[FieldOffset(0)]
		public IntPtr xmapping;
		[FieldOffset(0)]
		public IntPtr xerror;
		[FieldOffset(0)]
		public IntPtr xkeymap;
		[FieldOffset(0)]
		public IntPtr pad;
	}
	
	[StructLayout(LayoutKind.Sequential)]
	internal struct XKeyEvent
	{
		public XEventName type;
		public IntPtr serial;
		public bool send_event;
		public IntPtr display;
		public IntPtr window;
		public IntPtr root;
		public IntPtr subwindow;
		public IntPtr time;
		public int x;
		public int y;
		public int x_root;
		public int y_root;
		public int state;
		public int keycode;
		public bool same_screen;
	}
	
	[StructLayout(LayoutKind.Sequential)]
	internal struct XMotionEvent
	{
		public XEventName type;
		public IntPtr serial;
		public bool send_event;
		public IntPtr display;
		public IntPtr window;
		public IntPtr root;
		public IntPtr subwindow;
		public IntPtr time;
		public int x;
		public int y;
		public int x_root;
		public int y_root;
		public int state;
		public byte is_hint;
		public bool same_screen;
	}
	
	[StructLayout(LayoutKind.Sequential)]
	internal struct XButtonEvent
	{
		public XEventName type;
		public IntPtr serial;
		public bool send_event;
		public IntPtr display;
		public IntPtr window;
		public IntPtr root;
		public IntPtr subwindow;
		public IntPtr time;
		public int x;
		public int y;
		public int x_root;
		public int y_root;
		public int state;
		public int button;
		public bool same_screen;
	}
	
	[StructLayout(LayoutKind.Sequential)]
	internal struct XKeyboardState 
	{
		public int key_click_percent;
		public int bell_percent;
		public uint bell_pitch, bell_duration;
		public uint led_mask;
		public int global_auto_repeat;
		public AutoRepeats auto_repeats;

		[StructLayout(LayoutKind.Explicit)]
		public struct AutoRepeats 
		{
			[FieldOffset (0)]
			public byte first;				
			[FieldOffset (31)]
			public byte last;
		}
	}
}
