using System;
using System.Runtime.InteropServices;
using System.Text;
using System.Collections.Generic;

namespace AC.Input.Platforms.Linux
{
	internal class LinuxContext : BaseContext
	{
		#region Constants
		private const string X11Lib = "libX11";
		
		private const int BadWindow = 3;
		private const int BadAccess = 10;
		
		private const int GrabModeAsync = 1;
						
		private const int AutoRepeatModeOff = 0;
		private const int AutoRepeatModeOn = 1;
		private const int AutoRepeatModeDefault = 2;
		
		
		private const long CurrentTime = 0;
		private const int ButtonPressMask = 1 << 2;
		private const int ButtonReleaseMask = 1 << 3;
		private const int EnterWindowMask = 1 << 4;
		private const int LeaveWindowMask = 1 << 5;
		private const int PointerMotionMask = 1 << 6;
		private const int PointerMotionHintMask = 1 << 7;
		private const int Mod4Mask = (1 << 6);
		private const int Mod1Mask = ( 1 << 3 );
		private const int MouseMovedPressedReleased = Mod4Mask | ButtonReleaseMask | ButtonPressMask;
		private const long KeyPressMask = ( 1L << 0 );
		private const long KeyReleaseMask = ( 1L << 1 );
		private const int ShiftMask = ( 1 << 0 );
		private const int LockMask = ( 1 << 1 );
		#endregion

		#region XKeyboardState
		[StructLayout(LayoutKind.Sequential)]
		internal struct XKeyboardState
		{
			public int key_click_percent;
			public int bell_percent;
			public uint bell_pitch, bell_duration;
			public uint led_mask;
			public int global_auto_repeat;
			public AutoRepeats auto_repeats;

			[StructLayout(LayoutKind.Explicit)]
			public struct AutoRepeats
			{
				[FieldOffset(0)]
				public byte first;
				[FieldOffset(31)]
				public byte last;
			}
		}
		#endregion

		#region KeySym (helper enum)
		public enum KeySym
		{
			XK_0 = 0x0030,  /* U+0030 DIGIT ZERO */
			XK_1 = 0x0031, /* U+0031 DIGIT ONE */
			XK_2 = 0x0032, /* U+0032 DIGIT TWO */
			XK_3 = 0x0033, /* U+0033 DIGIT THREE */
			XK_4 = 0x0034, /* U+0034 DIGIT FOUR */
			XK_5 = 0x0035, /* U+0035 DIGIT FIVE */
			XK_6 = 0x0036, /* U+0036 DIGIT SIX */
			XK_7 = 0x0037, /* U+0037 DIGIT SEVEN */
			XK_8 = 0x0038, /* U+0038 DIGIT EIGHT */
			XK_9 = 0x0039, /* U+0039 DIGIT NINE */
			XK_backSpace = 0xff08, /* Back space, back char */
			XK_minus = 0x002d,  /* U+002D HYPHEN-MINUS */
			XK_equal = 0x003d, /* U+003D EQUALS SIGN */
			XK_space = 0x0020, /* U+0020 SPACE */
			XK_comma = 0x002c, /* U+002C COMMA */
			XK_period = 0x002e, /* U+002E FULL STOP */
			XK_backslash = 0x005c, /* U+005C REVERSE SOLIDUS */
			XK_slash = 0x002f, /* U+002F SOLIDUS */
			XK_bracketleft = 0x005b, /* U+005B LEFT SQUARE BRACKET */
			XK_bracketright = 0x005d, /* U+005D RIGHT SQUARE BRACKET */
			XK_Escape = 0xff1b,
			XK_Caps_Lock = 0xffe5, /* Caps lock */
			XK_Tab = 0xff09,
			XK_Return = 0xff0d, /* Return, enter */
			XK_Control_L = 0xffe3, /* Left control */
			XK_Control_R = 0xffe4, /* Right control */
			XK_colon = 0x003a, /* U+003A COLON */
			XK_semicolon = 0x003b, /* U+003B SEMICOLON */
			XK_apostrophe = 0x0027, /* U+0027 APOSTROPHE */
			XK_grave = 0x0060, /* U+0060 GRAVE ACCENT */
			XK_a = 0x0061, /* U+0061 LATIN SMALL LETTER A */
			XK_b = 0x0062, /* U+0062 LATIN SMALL LETTER B */
			XK_c = 0x0063, /* U+0063 LATIN SMALL LETTER C */
			XK_d = 0x0064, /* U+0064 LATIN SMALL LETTER D */
			XK_e = 0x0065, /* U+0065 LATIN SMALL LETTER E */
			XK_f = 0x0066, /* U+0066 LATIN SMALL LETTER F */
			XK_g = 0x0067, /* U+0067 LATIN SMALL LETTER G */
			XK_h = 0x0068, /* U+0068 LATIN SMALL LETTER H */
			XK_i = 0x0069, /* U+0069 LATIN SMALL LETTER I */
			XK_j = 0x006a, /* U+006A LATIN SMALL LETTER J */
			XK_k = 0x006b, /* U+006B LATIN SMALL LETTER K */
			XK_l = 0x006c, /* U+006C LATIN SMALL LETTER L */
			XK_m = 0x006d, /* U+006D LATIN SMALL LETTER M */
			XK_n = 0x006e, /* U+006E LATIN SMALL LETTER N */
			XK_o = 0x006f, /* U+006F LATIN SMALL LETTER O */
			XK_p = 0x0070, /* U+0070 LATIN SMALL LETTER P */
			XK_q = 0x0071, /* U+0071 LATIN SMALL LETTER Q */
			XK_r = 0x0072, /* U+0072 LATIN SMALL LETTER R */
			XK_s = 0x0073, /* U+0073 LATIN SMALL LETTER S */
			XK_t = 0x0074, /* U+0074 LATIN SMALL LETTER T */
			XK_u = 0x0075, /* U+0075 LATIN SMALL LETTER U */
			XK_v = 0x0076, /* U+0076 LATIN SMALL LETTER V */
			XK_w = 0x0077, /* U+0077 LATIN SMALL LETTER W */
			XK_x = 0x0078, /* U+0078 LATIN SMALL LETTER X */
			XK_y = 0x0079, /* U+0079 LATIN SMALL LETTER Y */
			XK_z = 0x007a, /* U+007A LATIN SMALL LETTER Z */
			XK_F1 = 0xffbe,
			XK_F2 = 0xffbf,
			XK_F3 = 0xffc0,
			XK_F4 = 0xffc1,
			XK_F5 = 0xffc2,
			XK_F6 = 0xffc3,
			XK_F7 = 0xffc4,
			XK_F8 = 0xffc5,
			XK_F9 = 0xffc6,
			XK_F10 = 0xffc7,
			XK_F11 = 0xffc8,
			XK_F12 = 0xffc9,
			XK_F13 = 0xffca,
			XK_F14 = 0xffcb,
			XK_F15 = 0xffcc,
			XK_KP_0 = 0xffb0,
			XK_KP_1 = 0xffb1,
			XK_KP_2 = 0xffb2,
			XK_KP_3 = 0xffb3,
			XK_KP_4 = 0xffb4,
			XK_KP_5 = 0xffb5,
			XK_KP_6 = 0xffb6,
			XK_KP_7 = 0xffb7,
			XK_KP_8 = 0xffb8,
			XK_KP_9 = 0xffb9,
			XK_KP_Add = 0xffab,
			XK_KP_Subtract = 0xffad,
			XK_KP_Decimal = 0xffae,
			XK_KP_Equal = 0xffbd, /* Equals */
			XK_KP_Divide = 0xffaf,
			XK_KP_Multiply = 0xffaa,
			XK_KP_Enter = 0xff8d, /* Enter */
			XK_KP_Home = 0xff95,
			XK_KP_Up = 0xff97,
			XK_KP_Page_Up = 0xff9a,
			XK_KP_Left = 0xff96,
			XK_KP_Begin = 0xff9d,
			XK_KP_Right = 0xff98,
			XK_KP_End = 0xff9c,
			XK_KP_Down = 0xff99,
			XK_KP_Page_Down = 0xff9b,
			XK_KP_Insert = 0xff9e,
			XK_KP_Delete = 0xff9f,
			XK_Left = 0xff51, /* Move left, left arrow */
			XK_Up = 0xff52, /* Move up, up arrow */
			XK_Right = 0xff53, /* Move right, right arrow */
			XK_Down = 0xff54, /* Move down, down arrow */
			XK_Page_Up = 0xff55,
			XK_Page_Down = 0xff56,
			XK_Home = 0xff50,
			XK_End = 0xff57, /* EOL */
			XK_Num_Lock = 0xff7f,
			XK_Print = 0xff61,
			XK_Scroll_Lock = 0xff14,
			XK_Pause = 0xff13, /* Pause, hold */
			XK_Shift_L = 0xffe1, /* Left shift */
			XK_Shift_R = 0xffe2, /* Right shift */
			XK_Alt_R = 0xffea, /* Right alt */
			XK_Alt_L = 0xffe9, /* Left alt */
			XK_Insert = 0xff63, /* Insert, insert here */
			XK_Delete = 0xffff, /* Delete, rubout */
			XK_Super_L = 0xffeb, /* Left super */
			XK_Super_R = 0xffec, /* Right super */
			XK_Menu = 0xff67,
		}
		#endregion

		#region Buttons (helper enum)
		[Flags]
		private enum Buttons
		{		
			Button1Mask =	(1<<8),
			Button2Mask =	(1<<9),
			Button3Mask =	(1<<10),
			Button4Mask =	(1<<11),
			Button5Mask =	(1<<12),
			Button1MotionMask =	(1<<8),
			Button2MotionMask =	(1<<9),
			Button3MotionMask =	(1<<10),
			Button4MotionMask =	(1<<11),
			Button5MotionMask =	(1<<12),
			ShiftMask =	(1<<0),
			LockMask =	(1<<1),
			ControlMask =	(1<<2),
			Mod1Mask =	(1<<3),
			Mod2Mask =	(1<<4),
			Mod3Mask =	(1<<5),
			Mod4Mask =	(1<<6),
			Mod5Mask =	(1<<7),
			Button1 =	1,
			Button2 =	2,
			Button3 =	3,
			Button4 =	4,
			Button5 =	5,
		}
		#endregion

		#region System Invokes
		[DllImport(X11Lib, EntryPoint = "XPending")]
		private static extern int XPending(IntPtr diplay);

		[DllImport(X11Lib, EntryPoint = "XPeekEvent")]
		private static extern void XPeekEvent(IntPtr display, ref XEvent xevent);

		[DllImport(X11Lib, EntryPoint = "XNextEvent")]
		private static extern IntPtr XNextEvent(IntPtr display, ref XEvent xevent);

		[DllImport(X11Lib, EntryPoint = "XNextEvent")]
		private static extern IntPtr XNextEvent(IntPtr display, ref XKeyEvent xevent);

		[DllImport(X11Lib)]
		public static extern int XGrabPointer(IntPtr display, IntPtr grabWindow,
			bool ownerMvents, int eventMask, int pointerMode, int keyboardMode,
			IntPtr confineTo, IntPtr cursor, long time);

		[DllImport(X11Lib)]
		public static extern int XUngrabPointer(IntPtr display, long time);

		[DllImport(X11Lib)]
		public static extern int XLookupString(ref XKeyEvent even, StringBuilder bufferReturn,
		    int bytesBuffer, out int keysymReturn, IntPtr xComposeStatus);

		[DllImport(X11Lib)]
		public static extern IntPtr XOpenDisplay(IntPtr displayName);

		[DllImport(X11Lib)]
		public static extern void XWarpPointer(IntPtr display, IntPtr sourceWindow,
			IntPtr destWindow, int srcX, int srcY, int srcWidth, int srcHeight,
			int destX, int destY);

		[DllImport(X11Lib)]
		public static extern bool XQueryPointer(IntPtr display, IntPtr window,
			out IntPtr rootWindow, out IntPtr childWindow, out int root_x_return,
			out int root_y_return, out int win_x_return, out int win_y_return,
			out uint mask_return);

		[DllImport(X11Lib)]
		public static extern IntPtr XCloseDisplay(IntPtr display);

		[DllImport(X11Lib)]
		public static extern void XUngrabKeyboard(IntPtr display, long currentTime);

		[DllImport(X11Lib)]
		public static extern int XGrabKeyboard(IntPtr display, IntPtr window,
			bool own, int pointerMode, int keyboardMode, long currentTime);

		[DllImport(X11Lib)]
		public static extern int XSelectInput(IntPtr display, IntPtr window, long mask);

		[DllImport(X11Lib)]
		public static extern bool XCheckMaskEvent(IntPtr display, int even_mask,
			ref XEvent even);

		[DllImport(X11Lib)]
		public static extern bool XkbSetDetectableAutoRepeat(IntPtr display,
			bool detectable, ref bool supported);

		[DllImport(X11Lib)]
		public static extern uint XKeycodeToKeysym(IntPtr display, int keycode,
			int index);

		[DllImport(X11Lib)]
		public static extern bool XTranslateCoordinates(IntPtr display, IntPtr src_w,
			IntPtr dest_w, int src_x, int src_y, out int dest_x_return,
			out int dest_y_return, out IntPtr child_return);

		[DllImport(X11Lib)]
		public static extern int XAutoRepeatOn(IntPtr display);

		[DllImport(X11Lib)]
		public static extern int XAutoRepeatOff(IntPtr display);

		[DllImport(X11Lib)]
		public static extern int XGetKeyboardControl(IntPtr display,
			out XKeyboardState old);
		#endregion

		#region Private
		private IntPtr displayHandle;
		private IntPtr lastWindowHandle;
		
		private int cacheX;
		private int cacheY;
		
		private Buttons lastMask;
		#endregion
		
		#region Public
		public List<Keys> PressedKeys
		{
			get;
			private set;
		}

		public List<Keys> ReleasedKeys
		{
			get;
			private set;
		}
		
		public Queue<bool> LeftButton
		{
			get;
			private set;
		}

		public Queue<bool> MiddleButton
		{
			get;
			private set;
		}

		public Queue<bool> RightButton
		{
			get;
			private set;
		}

		public Queue<bool> X1Button
		{
			get;
			private set;
		}

		public Queue<bool> X2Button
		{
			get;
			private set;
		}
		#endregion

		#region Constructor
		public LinuxContext()
		{
			displayHandle = XOpenDisplay(IntPtr.Zero);
			
			PressedKeys = new List<Keys>();
			ReleasedKeys = new List<Keys>();
			
			LeftButton = new Queue<bool>();
			MiddleButton = new Queue<bool>();
			RightButton = new Queue<bool>();
			X1Button = new Queue<bool>();
			X2Button = new Queue<bool>();
		}

		~LinuxContext()
		{
			XUngrabKeyboard(displayHandle, 0);
			XCloseDisplay(displayHandle);
		}
		#endregion

		#region Update
		public override void Update()
		{
			if(WindowHandle == IntPtr.Zero)
			{
				return;
			}
			
			while(XPending(displayHandle) > 0)
			{
				XKeyEvent e = new XKeyEvent();
				XNextEvent(displayHandle, ref e);
				
				int key = 0;
				StringBuilder sb = new StringBuilder(6);
				XLookupString(ref e, sb, 6, out key, IntPtr.Zero);
				if(e.type == XEventName.KeyPress)
				{
					Keys acKey = X11Helper.ConvertX11ToKeys((uint)key);
					if(ReleasedKeys.Contains(acKey))
					{
						ReleasedKeys.Remove(acKey);
					}
					PressedKeys.Add(acKey);
				}
				else if(e.type == XEventName.KeyRelease)
				{
					ReleasedKeys.Add(X11Helper.ConvertX11ToKeys((uint)key));
				}
			}

			#region Mouse
			IntPtr w2, w3;
			int x2, y2;
			uint mask;
			XQueryPointer(displayHandle, WindowHandle, out w2, out w3, out x2,
				out y2, out cacheX, out cacheY, out mask);
			
			Buttons buttonsMask = (Buttons)mask;
			if((buttonsMask & Buttons.Button1Mask) != 0)
			{
				LeftButton.Enqueue(true);
			}
			else if((lastMask & Buttons.Button1Mask) != 0)
			{
				LeftButton.Enqueue(false);
			}
			
			if((buttonsMask & Buttons.Button2Mask) != 0)
			{
				MiddleButton.Enqueue(true);
			}
			else if((lastMask & Buttons.Button2Mask) != 0)
			{
				MiddleButton.Enqueue(false);
			}
			
			if((buttonsMask & Buttons.Button3Mask) != 0)
			{
				RightButton.Enqueue(true);
			}
			else if((lastMask & Buttons.Button3Mask) != 0)
			{
				RightButton.Enqueue(false);
			}
			
			if((buttonsMask & Buttons.Button4Mask) != 0)
			{
				X1Button.Enqueue(true);
			}
			else if((lastMask & Buttons.Button4Mask) != 0)
			{
				X1Button.Enqueue(false);
			}
			
			if((buttonsMask & Buttons.Button5Mask) != 0)
			{
				X2Button.Enqueue(true);
			}
			else if((lastMask & Buttons.Button5Mask) != 0)
			{
				X2Button.Enqueue(false);
			}
			
			lastMask = buttonsMask;
			#endregion
		}
		#endregion

		#region CheckGamePadConnection
		public override bool CheckGamePadConnection(int index)
		{
			return false;
		}
		#endregion

		#region GetPressedKeys
		public override Keys[] GetPressedKeys()
		{
			Keys[] result;

			lock (PressedKeys)
			{
				result = PressedKeys.ToArray();
				PressedKeys.Clear();
			}
			
			return result;
		}
		#endregion
		
		#region GetReleasedKeys
		public override Keys[] GetReleasedKeys()
		{
			Keys[] result;

			lock (ReleasedKeys)
			{
				result = ReleasedKeys.ToArray();
				ReleasedKeys.Clear();
			}
			
			return result;
		}
		#endregion
		
		#region SetGamePadValues
		public override void SetGamePadValues(int index,
			out float thumbStickLeftX, out float thumbStickLeftY,
			out float thumbStickRightX, out float thumbStickRightY,
			out float triggerLeft, out float triggerRight,
			ref bool[] dpadButtons, ref bool[] buttons)
		{
			thumbStickLeftX = 0f;
			thumbStickLeftY = 0f;
			thumbStickRightX = 0f;
			thumbStickRightY = 0f;
			triggerLeft = 0f;
			triggerRight = 0f;
		}
		#endregion
		
		#region SetMousePosition
		public override void SetMousePosition(int x, int y)
		{
			XWarpPointer(displayHandle, WindowHandle, WindowHandle, 0, 0, 0, 0, x, y);
		}
		#endregion

		#region SetMouseValues
		public override void SetMouseValues(out int X, out int Y,
			out bool[] pressed, out bool[] released,
			out int scrollMovement)
		{
			X = cacheX < 0 ? 0 : cacheX;
			Y = cacheY < 0 ? 0 : cacheY;
			
			pressed = new bool[5];
			released = new bool[5];
			
			SetMouseButtonState(ref pressed[0], ref released[0], LeftButton);
			SetMouseButtonState(ref pressed[1], ref released[1], MiddleButton);
			SetMouseButtonState(ref pressed[2], ref released[2], RightButton);
			SetMouseButtonState(ref pressed[3], ref released[3], X1Button);
			SetMouseButtonState(ref pressed[4], ref released[4], X2Button);
			
			scrollMovement = 0;
		}
		#endregion

		#region SetMouseButtonState
		private void SetMouseButtonState(ref bool pressed, ref bool released,
			Queue<bool> queue)
		{
			if (queue.Count > 0)
			{
				if (queue.Dequeue())
				{
					pressed = true;
				}
				else
				{
					released = true;
				}
			}
		}
		#endregion
		
		#region WindowHandleChanged
		protected override void WindowHandleChanged()
		{
			if(lastWindowHandle != IntPtr.Zero)
			{
				XUngrabKeyboard(displayHandle, CurrentTime);
			}
							
			lastWindowHandle = WindowHandle;
			if(XSelectInput(displayHandle, WindowHandle, KeyPressMask | KeyReleaseMask) == BadWindow)
			{
				throw new Exception("Failed to select XInput!");
			}
			
			XGrabKeyboard(displayHandle, WindowHandle, true, GrabModeAsync, GrabModeAsync, CurrentTime);
			//XGrabPointer(displayHandle, WindowHandle, false, 0, GrabModeAsync, GrabModeAsync, WindowHandle, IntPtr.Zero, CurrentTime);
		}
		#endregion
	}
}
