﻿using System;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace AC.Input.Platforms
{
	public static class Platform
	{
		#region Public
		public static bool OnWindows
		{
			get;
			private set;
		}
		public static bool OnMacOS
		{
			get;
			private set;
		}
		public static bool OnLinux
		{
			get;
			private set;
		}
		#endregion

		#region Constructor
		static Platform()
		{
			PlatformID id = Environment.OSVersion.Platform;
			if (id == PlatformID.Win32NT ||
				id == PlatformID.Win32S ||
				id == PlatformID.Win32Windows ||
				id == PlatformID.WinCE)
			{
				OnWindows = true;
			}
			else if (id == PlatformID.Unix ||
				id == (PlatformID)4)
			{
				utsname uts = new utsname();
				uname(out uts);
				switch (uts.sysname)
				{
					default:
						throw new PlatformNotSupportedException("Unknown platform.");

					case "Linux":
						OnLinux = true;
						break;

					case "Darwin":
						OnMacOS = true;
						break;
				}
			}
			else
			{
				throw new PlatformNotSupportedException("Unknown platform.");
			}
		}
		#endregion

		#region Detect Unix Kernel
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
		struct utsname
		{
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
			public string sysname;

			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
			public string nodename;

			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
			public string release;

			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
			public string version;

			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
			public string machine;

			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 1024)]
			public string extraJustInCase;
		}

		[DllImport("libc")]
		private static extern void uname(out utsname uname_struct);

		#endregion
	}
}
