﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace AC.Input.Platforms.Windows
{
	/// <summary>
	/// http://msdn.microsoft.com/en-us/library/ms644984%28v=vs.85%29.aspx
	/// 
	/// http://msdn.microsoft.com/en-us/library/ms646267%28v=vs.85%29.aspx#_win32_Keystroke_Message_Flags
	/// 
	/// http://www.java2s.com/Open-Source/CSharp/Media/mpwX/iFun/Native/Win32Keyboard.cs.htm
	/// </summary>
	internal class KeyboardHook : WindowsHook
	{
		#region Public
		public List<Keys> PressedKeys
		{
			get;
			private set;
		}

		public List<Keys> ReleasedKeys
		{
			get;
			private set;
		}
		#endregion

		#region Constructor
		public KeyboardHook()
			: base(HookType.WH_KEYBOARD)
		{
			PressedKeys = new List<Keys>();
			ReleasedKeys = new List<Keys>();
		}
		#endregion

		#region HandleProcMessage
		protected override void HandleProcMessage(int code, IntPtr wParam,
			IntPtr lParam)
		{
			if (code == HC_ACTION)
			{
				Keys keyCode = (Keys)wParam.ToInt32();
				bool isKeyDown = (uint)(lParam.ToInt32() & 0x80000000) >> 0xFF != 1;
				if (isKeyDown)
				{
					PressedKeys.Add(keyCode);
				}
				else
				{
					ReleasedKeys.Add(keyCode);
				}
			}
		}
		#endregion
	}
}
