using System;
using System.Runtime.InteropServices;

namespace AC.Input.Platforms.Windows
{
	/// <summary>
	/// http://msdn.microsoft.com/en-us/library/ms724506%28VS.85%29.aspx
	/// </summary>
	internal static class WindowHelper
	{
		#region NONCLIENTMETRICS (helper struct)
		[StructLayout(LayoutKind.Explicit)]
		public struct NONCLIENTMETRICS
		{
			[FieldOffset(0)]
			public uint cbSize;
			[FieldOffset(4)]
			public int iBorderWidth;
			[FieldOffset(8)]
			public int iScrollWidth;
			[FieldOffset(12)]
			public int iScrollHeight;
			[FieldOffset(16)]
			public int iCaptionWidth;
			[FieldOffset(20)]
			public int iCaptionHeight;
			[FieldOffset(24)]
			public LOGFONT lfCaptionFont;
			[FieldOffset(84)]
			public int iSmCaptionWidth;
			[FieldOffset(88)]
			public int iSmCaptionHeight;
			[FieldOffset(92)]
			public LOGFONT lfSmCaptionFont;
			[FieldOffset(152)]
			public int iMenuWidth;
			[FieldOffset(156)]
			public int iMenuHeight;
			[FieldOffset(160)]
			public LOGFONT lfMenuFont;
			[FieldOffset(220)]
			public LOGFONT lfStatusFont;
			[FieldOffset(280)]
			public LOGFONT lfMessageFont;
		}
		#endregion

		#region NONCLIENTMETRICSEX (helper struct)
		public struct NONCLIENTMETRICSEX
		{
			public NONCLIENTMETRICS StandardMetrics;
			public int iPaddedBorderWidth;
		}
		#endregion

		#region LOGFONT (helper struct)
		[StructLayout(LayoutKind.Sequential)]
		public struct LOGFONT
		{
			public int lfHeight;
			public int lfWidth;
			public int lfEscapement;
			public int lfOrientation;
			public int lfWeight;
			public byte lfItalic;
			public byte lfUnderline;
			public byte lfStrikeOut;
			public byte lfCharSet;
			public byte lfOutPrecision;
			public byte lfClipPrecision;
			public byte lfQuality;
			public byte lfPitchAndFamily;
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
			public string lfFaceName;
		}
		#endregion

		#region Constants
		private const int SPI_GETNONCLIENTMETRICS = 0x0029;
		private static uint MetricsSize;
		#endregion

		#region Private
		public static NONCLIENTMETRICS Metrics;
		public static NONCLIENTMETRICSEX MetricsEx;
		private static bool useExtendedMetrics;
		#endregion

		#region Constructor
		static WindowHelper()
		{
			useExtendedMetrics = Environment.OSVersion.Version.Major >= 6;
			MetricsSize = (uint)(Marshal.SizeOf(typeof(NONCLIENTMETRICS)) +
				(useExtendedMetrics ? 4 : 0));

			Metrics = new NONCLIENTMETRICS();
			MetricsEx = new NONCLIENTMETRICSEX();
		}
		#endregion

		#region UpdateMetrics
		public static void UpdateMetrics()
		{
			if (useExtendedMetrics)
			{
				MetricsEx = new NONCLIENTMETRICSEX();
				MetricsEx.StandardMetrics.cbSize = MetricsSize;
				SystemParametersInfoEx(SPI_GETNONCLIENTMETRICS, MetricsSize, ref MetricsEx, 0);
				Metrics = MetricsEx.StandardMetrics;
			}
			else
			{
				Metrics = new NONCLIENTMETRICS();
				Metrics.cbSize = MetricsSize;
				SystemParametersInfo(SPI_GETNONCLIENTMETRICS, MetricsSize, ref Metrics, 0);
				MetricsEx.StandardMetrics = Metrics;
			}
		}
		#endregion

		#region SystemParametersInfo
		[DllImport("user32.dll", EntryPoint = "SystemParametersInfo")]
		private static extern bool SystemParametersInfo(int action, uint intParam,
			ref NONCLIENTMETRICS metrics, int update);

		[DllImport("user32.dll", EntryPoint = "SystemParametersInfo")]
		private static extern bool SystemParametersInfoEx(int action, uint intParam,
			ref NONCLIENTMETRICSEX metrics, int update);
		#endregion
	}
}
