﻿using System;
using System.Runtime.InteropServices;

namespace AC.Input.Platforms.Windows
{
	/// <summary>
	/// http://support.microsoft.com/kb/Q318804
	/// </summary>
	internal abstract class WindowsHook : IDisposable
	{
		private delegate int HookProc(int code, IntPtr wParam, IntPtr lParam);

		#region Constants
		public const int HC_ACTION = 0;
		#endregion

		#region Private
		private IntPtr hookHandle;
		private HookType hookType;
		private HookProc callback;
		#endregion

		#region Constructor
		public WindowsHook(HookType hook)
		{
			hookType = hook;
			callback = MessageCallback;
			hookHandle = SetWindowsHookEx(hookType, callback, IntPtr.Zero,
				(int)AppDomain.GetCurrentThreadId());
		}

		~WindowsHook()
		{
			Dispose();
		}
		#endregion

		#region Dispose
		public void Dispose()
		{
			UnhookWindowsHookEx(hookHandle);
			hookHandle = IntPtr.Zero;
		}
		#endregion

		#region MessageCallback
		private int MessageCallback(int code, IntPtr wParam, IntPtr lParam)
		{
			if (code >= 0)
			{
				HandleProcMessage(code, wParam, lParam);
			}

			return CallNextHookEx(hookHandle, code, wParam, lParam);
		}
		#endregion

		#region HandleProcMessage (abstract)
		protected abstract void HandleProcMessage(int code, IntPtr wParam,
			IntPtr lParam);
		#endregion

		#region Native Invokes
		[DllImport("user32.dll")]
		private static extern IntPtr SetWindowsHookEx(HookType code,
			HookProc func, IntPtr hInstance, int threadID);

		[DllImport("user32.dll")]
		private static extern int UnhookWindowsHookEx(IntPtr hhook);

		[DllImport("user32.dll")]
		private static extern int CallNextHookEx(IntPtr hhook,
			int code, IntPtr wParam, IntPtr lParam);
		#endregion
	}
}
