﻿using System;
using System.Windows.Forms;
using AC.Input;
using System.Drawing;

namespace TestApp
{
	public class GamePadPanel : Panel
	{
		private GamePad gamePad;

		private Pen blackPen = new Pen(Color.Black);
		private Brush grayBrush = new SolidBrush(Color.Gray);

		public GamePadPanel()
			: base()
		{
			DoubleBuffered = true;
		}

		public void Update(GamePad setGamePad)
		{
			gamePad = setGamePad;
			Refresh();
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			base.OnPaint(e);

			if (gamePad == null)
			{
				return;
			}

			Graphics graph = e.Graphics;

			#region Trigger
			{
				int triggerHeight = 25;
				int triggerLeftHeight = (int)((1f - gamePad.TriggerLeft) * triggerHeight);
				int triggerRightHeight = (int)((1f - gamePad.TriggerRight) * triggerHeight);

				graph.FillRectangle(grayBrush, new Rectangle(20,
					triggerHeight - triggerLeftHeight, 10, triggerLeftHeight));
				graph.DrawRectangle(blackPen, new Rectangle(20, 0, 10, triggerHeight));

				graph.FillRectangle(grayBrush, new Rectangle(Width - 30,
					triggerHeight - triggerRightHeight, 10, triggerRightHeight));
				graph.DrawRectangle(blackPen, new Rectangle(Width - 30, 0, 10,
					triggerHeight));
			}
			#endregion

			#region Buttons
			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.Buttons.ShoulderLeft)),
				new Rectangle(5, 30, 50, 10));
			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.Buttons.ShoulderRight)),
				new Rectangle(Width - 55, 30, 50, 10));

			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.Buttons.Y)),
				new Rectangle(Width - 45, 45, 15, 15));
			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.Buttons.X)),
				new Rectangle(Width - 60, 60, 15, 15));
			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.Buttons.B)),
				new Rectangle(Width - 30, 60, 15, 15));
			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.Buttons.A)),
				new Rectangle(Width - 45, 75, 15, 15));

			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.Buttons.Start)),
				new Rectangle((Width / 2) + 20, 62, 10, 10));
			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.Buttons.Back)),
				new Rectangle((Width / 2) - 30, 62, 10, 10));
			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.Buttons.Big)),
				new Rectangle((Width / 2) - 12, 55, 25, 25));

			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.DPad.Top)),
				new Rectangle(45, Height - 50, 15, 15));
			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.DPad.Left)),
				new Rectangle(30, Height - 35, 15, 15));
			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.DPad.Right)),
				new Rectangle(60, Height - 35, 15, 15));
			graph.FillRectangle(
				new SolidBrush(GetColorByButtonState(gamePad.DPad.Bottom)),
				new Rectangle(45, Height - 20, 15, 15));
			#endregion

			#region ThumbSticks
			graph.DrawEllipse(blackPen, new Rectangle(10, 45, 45, 45));
			graph.DrawEllipse(blackPen, new Rectangle(Width - 85, Height - 50, 45, 45));

			int stickLeftX = (int)(gamePad.ThumbStickLeftX * 20f);
			int stickLeftY = (int)(gamePad.ThumbStickLeftY * 20f);
			graph.FillEllipse(
				new SolidBrush(GetColorByButtonState(gamePad.Buttons.ThumbStickLeft)),
				new Rectangle(22 + stickLeftX, 57 + stickLeftY, 20, 20));

			int stickRightX = (int)(gamePad.ThumbStickRightX * 20f);
			int stickRightY = (int)(gamePad.ThumbStickRightY * 20f);
			graph.FillEllipse(
				new SolidBrush(GetColorByButtonState(gamePad.Buttons.ThumbStickRight)),
				new Rectangle(Width - 73 + stickRightX, Height - 38 + stickRightY, 20, 20));
			#endregion
		}

		#region GetColorByButtonState
		private Color GetColorByButtonState(AC.Input.ButtonState state)
		{
			switch (state)
			{
				default:
				case AC.Input.ButtonState.NotPressed:
					return Color.Gray;

				case AC.Input.ButtonState.JustPressed:
					return Color.Yellow;

				case AC.Input.ButtonState.Pressed:
					return Color.Green;

				case AC.Input.ButtonState.Released:
					return Color.Red;
			}
		}
		#endregion
	}
}
