using System;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using AC.Input;

namespace TestApp
{
	public partial class MainWindow : Form
	{
		public MainWindow()
		{
			InitializeComponent();
		}
		
		protected override void OnActivated(EventArgs e)
		{
			base.OnActivated(e);
			InputManager.WindowHandle = Handle;
		}

		private void timer1_Tick(object sender, EventArgs e)
		{
			// If we wanna test a low fps scenario, we need to sleep some time.
			if (testLowFpsCheckBox.Checked)
			{
				Thread.Sleep(100);
			}

			InputManager.Run();

			#region Mouse
			UpdatePanelByButtonState(mouseLeftPanel, Mouse.Instance.LeftButton);
			UpdatePanelByButtonState(mouseMiddlePanel, Mouse.Instance.MiddleButton);
			UpdatePanelByButtonState(mouseRightPanel, Mouse.Instance.RightButton);
			UpdatePanelByButtonState(mouseX1Panel, Mouse.Instance.X1Button);
			UpdatePanelByButtonState(mouseX2Panel, Mouse.Instance.X2Button);

			mousePosLabel.Text = "X: " + Mouse.Instance.X + " Y: " + Mouse.Instance.Y;
			#endregion

			#region GamePad
			gamePadPanel1.Update(GamePad.One);
			gamePadPanel2.Update(GamePad.Two);
			gamePadPanel3.Update(GamePad.Three);
			gamePadPanel4.Update(GamePad.Four);
			#endregion

			#region Keyboard
			keyboardPressedLabel.Text = "A State: " +
				Keyboard.Instance.GetState(AC.Input.Keys.A);
			#endregion

			if (Keyboard.Instance.GetState(AC.Input.Keys.P) ==
				AC.Input.ButtonState.Released)
			{
				Mouse.Instance.SetPosition(10, 10);
			}
		}

		#region UpdatePanelByButtonState
		private void UpdatePanelByButtonState(Panel panel, AC.Input.ButtonState state)
		{
			switch (state)
			{
				case AC.Input.ButtonState.NotPressed:
					panel.BackColor = Color.Gray;
					break;
				case AC.Input.ButtonState.JustPressed:
					panel.BackColor = Color.Yellow;
					break;
				case AC.Input.ButtonState.Pressed:
					panel.BackColor = Color.Green;
					break;
				case AC.Input.ButtonState.Released:
					panel.BackColor = Color.Red;
					break;
			}
		}
		#endregion
	}
}
